==> gitlab-tools-v0.2.0.tar.gz: gitlab-tools/apis/install-api/make-package-api.sh <==
#!/bin/bash
#
# install-api:make-package-api.sh
#

if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 1
fi
if [ "$( type -t P )" != 'function' ] ; then
  echofatal 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function P not found.' >&2
  exit 1
fi
if ! is_variable_define_and_not_empty_safe TARGET_HOME ; then
  echofatal 'TARGET_HOME must be define.'
  exit 1
fi

function test_tag {
  local -r tag="$1"

  if ! git tag | grep -q "^${tag}$" ; then
    echofatal "Tag '${tag}' not found."
    git tag >&2
    exit 1
  fi
}

function get_remote_name {
  git remote show | head -n 1
}

function get_remote_url {
  git remote get-url "$( get_remote_name )"
}

function __checkout_and_prepare_release__ {
  local -r remote_url="${1}"
  local -r tag_name="${2}"
  local    directory=

  directory="$( basename "${remote_url%.git}" )"

  [ -d "${TARGET_HOME}" ] && rm -fr "${TARGET_HOME}"
  mkdir -v "${TARGET_HOME}" || exit 1
  pushd "${TARGET_HOME}" >/dev/null || exit 1

  git clone --recurse-submodules --branch "${tag_name}" "${remote_url}" "${directory}" || exit 1

  pushd "${directory}" >/dev/null || exit 1

  git checkout -b "package-${tag_name}" || exit 1

  echoinfo "Prepare package into: '${TARGET_HOME}'"
  rm -fr '.git/'
  rm '.gitignore'
  rm '.gitlab-ci.yml'
  rm '.gitmodules'

  popd >/dev/null || exit 1

  local -r release_basename="${tag_name}"

  tar -czvf "${release_basename}.tar.gz" "${directory}" || exit 1
  echoinfo "Now you need to upload: '${release_basename}.tar.gz'"

  popd >/dev/null || exit 1
}

function __do_prepare_release__ { # make_main
  local -r tag_name="$1"
  local    url=

  git fetch --verbose --prune || exit 1

  test_tag "${tag_name}"

  url="$( get_remote_url )"
  if [ -z "${url}" ] ; then
    echofatal "Can't found git remote URL."
    exit 1
  fi

  __checkout_and_prepare_release__ "${url}" "${tag_name}"
}


function make_usage {
  cat <<EOF
Checkout tag and create archive for release.

Usage:

  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" ) --tag TAGNAME

Options:
  -h,           --help  Display help message and exit.
  -T TAGNAME,   --tag TAGNAME
                        Prepare release for TAGNAME
  -L,           --list  List tags

EOF
}

function make_main {
  local param=
  local param_tag_name=

  pushd "${PROJECT_HOME}" >/dev/null || exit 1

  while [ $# -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help') make_usage ; exit 0 ;;
      '-T'|'--tag') param_tag_name="$( P "$@" )" && shift ;;
      '-L'|'--list') git fetch --all --quiet ; git tag || return 1 ; exit 0 ;;

      *)
        echofatal "Unknown option: '${param}'" >&2
        make_usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${param_tag_name}" ] ; then
    echofatal "--tag value is missing." >&2
    make_usage >&2
    exit 1
  fi

  __do_prepare_release__ "${param_tag_name}" || exit 1

  popd >/dev/null || exit 1
  echoinfo "$( realpath "$0" ) done."
}

==> gitlab-tools-v0.2.1.tar.gz: gitlab-tools/apis/install-api/make-package-api.sh <==
#!/bin/bash
#
# install-api:make-package-api.sh
#

if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 1
fi
if [ "$( type -t P )" != 'function' ] ; then
  echofatal 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function P not found.' >&2
  exit 1
fi
if ! is_variable_define_and_not_empty_safe TARGET_HOME ; then
  echofatal 'TARGET_HOME must be define.'
  exit 1
fi
if ! is_variable_define_and_not_empty_safe DIRECTORIES_TO_REMOVE_FROM_RELEASE ; then
  echowarn 'DIRECTORIES_TO_REMOVE_FROM_RELEASE is not define.'
  declare -gra DIRECTORIES_TO_REMOVE_FROM_RELEASE=()
fi


function test_tag {
  local -r tag="$1"

  if ! git tag | grep -q "^${tag}$" ; then
    echofatal "Tag '${tag}' not found."
    git tag >&2
    exit 1
  fi
}

function get_remote_name {
  git remote show | head -n 1
}

function get_remote_url {
  git remote get-url "$( get_remote_name )"
}

function __checkout_and_prepare_release__ {
  local -r remote_url="${1}"
  local -r tag_name="${2}"
  local    release_directory=

  release_directory="$( basename "${remote_url%.git}" )" || exit 1
  local -r release_directory

  [ -d "${TARGET_HOME}" ] && rm -fr "${TARGET_HOME}"
  mkdir -v "${TARGET_HOME}" || exit 1
  pushd "${TARGET_HOME}" >/dev/null || exit 1

  git clone --recurse-submodules --branch "${tag_name}" "${remote_url}" "${release_directory}" || exit 1

  pushd "${release_directory}" >/dev/null || exit 1

  git checkout -b "package-${tag_name}" || exit 1

  echoinfo "Prepare package into: '${TARGET_HOME}'"
  rm -fr '.git/'
  rm '.gitignore'
  rm '.gitlab-ci.yml'
  rm '.gitmodules'

  if [ "${#DIRECTORIES_TO_REMOVE_FROM_RELEASE[@]}" -gt 0 ] ; then
    local directory_to_remove=

    for directory_to_remove in "${DIRECTORIES_TO_REMOVE_FROM_RELEASE[@]}" ; do
      echoinfo "Removing directory: '${directory_to_remove}'"
      rm -fr "${directory_to_remove}"
    done
  fi

  popd >/dev/null || exit 1

  local -r release_basename="${tag_name}"

  tar -czvf "${release_basename}.tar.gz" "${release_directory}" || exit 1
  echoinfo "Now you need to upload: '${release_basename}.tar.gz'"
  rm -fr "${release_directory}" || exit 1

  popd >/dev/null || exit 1
}

function __do_prepare_release__ { # make_main
  local -r tag_name="$1"
  local    url=

  git fetch --verbose --prune || exit 1

  test_tag "${tag_name}"

  url="$( get_remote_url )"
  if [ -z "${url}" ] ; then
    echofatal "Can't found git remote URL."
    exit 1
  fi

  __checkout_and_prepare_release__ "${url}" "${tag_name}"
}


function make_usage {
  cat <<EOF
Checkout tag and create archive for release.

Usage:

  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" ) --tag TAGNAME

Options:
  -h,           --help  Display help message and exit.
  -T TAGNAME,   --tag TAGNAME
                        Prepare release for TAGNAME
  -L,           --list  List tags

EOF
}

function make_main {
  local param=
  local param_tag_name=

  pushd "${PROJECT_HOME}" >/dev/null || exit 1

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help') make_usage ; exit 0 ;;
      '-T'|'--tag') param_tag_name="$( P "$@" )" && shift ;;
      '-L'|'--list') git fetch --all --quiet ; git tag || return 1 ; exit 0 ;;

      *)
        echofatal "Unknown option: '${param}'" >&2
        make_usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${param_tag_name}" ] ; then
    echofatal "--tag value is missing." >&2
    make_usage >&2
    exit 1
  fi

  __do_prepare_release__ "${param_tag_name}" || exit 1

  popd >/dev/null || exit 1
  echoinfo "$( realpath "$0" ) done."
}
